#include <iostream>
#include <fstream>
#include "tmd.h"

using namespace std;

string makefilenamePDF(string Qtype){
  string filename;
  filename = "./PDFGrids/PDF"+Qtype+".m";
  return filename;
}
string makefilenameFF(string Qtype){
  string filename;
  filename = "./FFGrids/FF"+Qtype+".m";
  return filename;
}


int Locate(double xx[],int n,double x)
  // returns an integer j such that x lies inbetween xx[j] and xx[j+1].
  // unit offset of increasing ordered array xx assumed.
  // n is the length of the array (xx[n] highest element).
{
  int ju,jm,jl(0),j;
  ju=n+1;
  
  while (ju-jl>1) {
    jm=(ju+jl)/2; // compute a mid point.
    if ( x>= xx[jm])
      jl=jm;
    else ju=jm;
  }
  if (x==xx[1]) j=1;
  else if (x==xx[n]) j=n-1;
  else j=jl;
  
  return j;
}

double Polderivative1(double x1,double x2,double x3,double y1,double y2,double y3)
  // returns the estimate of the derivative at x1 obtained by a polynomial 
  // interpolation using the three points (x_i,y_i).
{
  return (x3*x3*(y1-y2)+2.0*x1*(x3*(-y1+y2)+x2*(y1-y3))+x2*x2*(-y1+y3)+x1*x1*(-y2+y3))/((x1-x2)*(x1-x3)*(x2-x3));
}

double Polderivative2(double x1,double x2,double x3,double y1,double y2,double y3)
  // returns the estimate of the derivative at x2 obtained by a polynomial 
  // interpolation using the three points (x_i,y_i).
{
  return (x3*x3*(y1-y2)-2.0*x2*(x3*(y1-y2)+x1*(y2-y3))+x2*x2*(y1-y3)+x1*x1*(y2-y3))/((x1-x2)*(x1-x3)*(x2-x3));
}

double Polderivative3(double x1,double x2,double x3,double y1,double y2,double y3)
  // returns the estimate of the derivative at x3 obtained by a polynomial 
  // interpolation using the three points (x_i,y_i).
{
  return (x3*x3*(-y1+y2)+2.0*x2*x3*(y1-y3)+x1*x1*(y2-y3)+x2*x2*(-y1+y3)+2.0*x1*x3*(-y2+y3))/((x1-x2)*(x1-x3)*(x2-x3));
}


 
void Ccoeffs(double y[],double y1[],double y2[],double y12[],double d1,double d2,double c[][5]){
								

/*
Given arrays y[1..4], y1[1..4], y2[1..4], and y12[1..4], containing the function gradients, and cross derivative at the four grid points of a rectangular gridcell (numbered counterclockwise from the lower left), and given d1 and d2, the length of the gridcell in the 1-and 
    2-directions, this routine returns the table c[1..4][1..4]that is used by the routine bicubint 
for bicubic interpolation. 
*/
 
static int wt[16][16]= 
{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0, 
-3,0,0,3,0,0,0,0,-2,0,0,-1,0,0,0,0, 
2,0,0,-2,0,0,0,0,1,0,0,1,0,0,0,0, 
0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0, 
0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0, 
0,0,0,0,-3,0,0,3,0,0,0,0,-2,0,0,-1, 
0,0,0,0,2,0,0,-2,0,0,0,0,1,0,0,1, 
-3,3,0,0,-2,-1,0,0,0,0,0,0,0,0,0,0, 
0,0,0,0,0,0,0,0,-3,3,0,0,-2,-1,0,0, 
9,-9,9,-9,6,3,-3,-6,6,-6,-3,3,4,2,1,2, 
-6,6,-6,6,-4,-2,2,4,-3,3,3,-3,-2,-1,-1,-2, 
2,-2,0,0,1,1,0,0,0,0,0,0,0,0,0,0, 
0,0,0,0,0,0,0,0,2,-2,0,0,1,1,0,0, 
-6,6,-6,6,-3,-3,3,3,-4,4,2,-2,-2,-2,-1,-1, 
4,-4,4,-4,2,2,-2,-2,2,-2,-2,2,1,1,1,1};	

int l,k,j,i;	

double xx,d1d2,cl[16],x[16];

d1d2=d1*d2;


for(i=1;i<=4;i++){ 
x[i-1]=y[i]; 
x[i+3]=y1[i]*d1; 
x[i+7]=y2[i]*d2; 
x[i+11]=y12[i]*d1d2; 
}

for(i=0;i<=15;i++){ 
xx=0.0; 
for(k=0;k<=15;k++) xx+=wt[i][k]*x[k]; 
cl[i]=xx; 
} 

l=0; 
for(i=1;i<=4;i++){
for(j=1;j<=4;j++) c[i][j]=cl[l++]; 
 }


}


 
double bicubint(double y[],double y1[],double y2[],double y12[],double x1l,double x1u,double x2l,double x2u,double x1,double x2){

//Given the function values, first derivatives and the cross derivatives
//at the grid points described by the Ccoeffs routine, with x1l x1u x2l x2u
//the lower and upper grid points in 1 and 2 directions this routine gives
//the function interpolation for points x1 and x2

int i,j;

double t,u,d1,d2,c[5][5];

for(i=0;i<=4;i++){
for(j=0;j<=4;j++) c[i][j]=0.0;
}


d1=x1u-x1l;
d2=x2u-x2l;

Ccoeffs(y,y1,y2,y12,d1,d2,c);


t=(x1-x1l)/d1; 
u=(x2-x2l)/d2;

double ansy=0.0;



for(i=4;i>=1;i--){ 
ansy=t*(ansy)+((c[i][4]*u+c[i][3])*u+c[i][2])*u+c[i][1]; 
}
 
return ansy;

} 


double PDFCollinear(string parton,double xA, double bT){

//Some error checks before going on
if(parton !="up" && parton !="down" && parton !="charm" && parton !="strange" && parton !="top" && parton !="bottom" && parton !="upbar" && parton !="downbar" && parton !="charmbar" && parton !="strangebar" && parton !="topbar" && parton !="bottombar" ){
cout << "You haven't entered a valid parton" << endl << "Choices are: up, upbar,down, downbar, charm, charmbar, strange, strangebar, top, topbar, down, downbar" <<endl ;
return -1.0;
}

if (xA<0.001 || xA > 0.69){
cout << "You haven't entered a valid range for xA. You must have 0.001 <= xA <= 0.69." << endl;
return -1.0;
}

if(bT<0.001 || bT>14.99){
cout << "You haven't entered a valid range for bT. You must have 0.001 <= bT <= 14.99." << endl;
return -1.0;
}


string filename=makefilenamePDF(parton);

int i,j,k,l;
double xx,btt;
char ch;

int nx = 113;
int nb = 1189;

int posx,posb;

int xGridcount=1;
int bGridcount=1;

double xGrid[nx+1];
double bGrid[nb+1];
double F[nx+1][nb+1];

double f[5],f1[5],f2[5],f12[5],f21[5];

double xincrement=0.0001;
double bincrement=0.0001;


//Preparing the x Grid
for(xx=0.001;xx<=0.7;xx=xx+xincrement){
xGrid[xGridcount]=xx;
xGridcount++;
if(xx>0.003) xincrement=0.0003;
if(xx>0.0098) xincrement=0.01;
}

//Preparing the bT Grid
for(btt=0.001;btt<=15;btt=btt+bincrement){
bGrid[bGridcount]=btt;
bGridcount++;
if(btt>0.09) bincrement=0.05;
}

//Reading out the quark data
ifstream myfile(filename.c_str());
myfile >> ch;
for(i=1;i<=nx;i++){
for(j=1;j<=nb;j++){
myfile >> F[i][j] >> ch ;
}
}
myfile.close();

//Locating the unit square grid positions for bicubic interpolation
posx = Locate(xGrid,nx,xA);
posb = Locate(bGrid,nb,bT);

//Calculating the values and the derivatives of the function on the 
//corners of the cube
//4 special cases where posx and posb are at the edges of the original grid

f[1]=F[posx][posb];
f[2]=F[posx+1][posb];
f[3]=F[posx+1][posb+1];
f[4]=F[posx][posb+1];

if(posx==1 && posb ==1){

f1[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f1[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);


f2[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f2[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f12[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f21[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);


}

else if(posx==nx-1 && posb==nb-1){

f1[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);

f1[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);


f2[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f2[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[3]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f12[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[1],f2[2]);

f12[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[1],f2[2]);

f12[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[4],f2[3]);

f12[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[4],f2[3]);

f21[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[1],f1[4]);

f21[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[2],f1[3]);

f21[3]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[2],f1[3]);

f21[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[1],f1[4]);

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);

}

else if(posx==1 && posb==nb-1) {

f1[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f1[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f2[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f2[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[3]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);



f12[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f21[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[1],f1[4]);

f21[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[2],f1[3]);

f21[3]=  Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[2],f1[3]);

f21[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[1],f1[4]);



f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);

}

else if(posx==nx-1 && posb==1){

f1[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);

f1[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);

f2[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f2[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);



f12[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[1],f2[2]);

f12[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[1],f2[2]);

f12[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[4],f2[3]);

f12[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[4],f2[3]);

f21[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f21[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f21[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f21[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);

}

else {

f1[1]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[2]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[3]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f1[4]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb]);

f2[1]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f2[2]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[3]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[4]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f12[1]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[1],f2[2]);

f12[2]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[3]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[4]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[4],f2[3]);

f21[1]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[1],f1[4]);

f21[2]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[2],f1[3]);

f21[3]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[4]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);


}



return bicubint(f,f1,f2,f12,xGrid[posx],xGrid[posx+1],bGrid[posb],bGrid[posb+1],xA,bT);
}


double FFCollinear(string parton,double xA, double bT){

//Some error checks before going on
if(parton !="up" && parton !="down" && parton !="charm"  && parton !="upbar" && parton !="downbar" && parton !="charmbar" ){
cout << "You haven't entered a valid parton" << endl << "Choices are: up, upbar,down, downbar, charm, charmbar" <<endl ;
return -1.0;
}

if (xA<0.2 || xA > 0.99){
cout << "You haven't entered a valid range for zA. You must have 0.2 <= zA <= 0.99." << endl;
return -1.0;
}

if(bT<0.005 || bT>14.99){
cout << "You haven't entered a valid range for bT. You must have 0.005 <= bT <= 14.99." << endl;
return -1.0;
}


string filename=makefilenameFF(parton);

int i,j,k,l;
double xx,btt;
char ch;

int nx = 140;
int nb = 1149;

int posx,posb;

int xGridcount=1;
int bGridcount=1;

double xGrid[nx+1];
double bGrid[nb+1];
double F[nx+1][nb+1];

double f[5],f1[5],f2[5],f12[5],f21[5];

double xincrement=0.005;
double bincrement=0.0001;


//Preparing the x Grid
for(xx=0.2;xx<=1.0;xx=xx+xincrement){
xGrid[xGridcount]=xx;
xGridcount++;
if(xx>0.8) xincrement=0.01;
}

//Preparing the bT Grid
for(btt=0.005;btt<=15;btt=btt+bincrement){
bGrid[bGridcount]=btt;
bGridcount++;
if(btt>0.09) bincrement=0.05;
}

//Reading out the quark data
ifstream myfile(filename.c_str());
myfile >> ch;
for(i=1;i<=nx;i++){
for(j=1;j<=nb;j++){
myfile >> F[i][j] >> ch ;
}
}
myfile.close();

//Locating the unit square grid positions for bicubic interpolation
posx = Locate(xGrid,nx,xA);
posb = Locate(bGrid,nb,bT);

//Calculating the values and the derivatives of the function on the 
//corners of the cube
//4 special cases where posx and posb are at the edges of the original grid

f[1]=F[posx][posb];
f[2]=F[posx+1][posb];
f[3]=F[posx+1][posb+1];
f[4]=F[posx][posb+1];

if(posx==1 && posb ==1){

f1[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f1[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);


f2[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f2[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f12[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f21[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);


}

else if(posx==nx-1 && posb==nb-1){

f1[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);

f1[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);


f2[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f2[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[3]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f12[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[1],f2[2]);

f12[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[1],f2[2]);

f12[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[4],f2[3]);

f12[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[4],f2[3]);

f21[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[1],f1[4]);

f21[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[2],f1[3]);

f21[3]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[2],f1[3]);

f21[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[1],f1[4]);

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);

}

else if(posx==1 && posb==nb-1) {

f1[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f1[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f2[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f2[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[3]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);



f12[1]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[2]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[3]= Polderivative2 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[4]= Polderivative1 (xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative3(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f21[1]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[1],f1[4]);

f21[2]= Polderivative2 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[2],f1[3]);

f21[3]=  Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[2],f1[3]);

f21[4]= Polderivative3 (bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative1(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[1],f1[4]);



f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);

}

else if(posx==nx-1 && posb==1){

f1[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);

f1[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb+1]);

f2[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f2[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);



f12[1]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[1],f2[2]);

f12[2]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative1(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[1],f2[2]);

f12[3]= Polderivative3 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[4],f2[3]);

f12[4]= Polderivative2 (xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[4],f2[3]);

f21[1]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f21[2]= Polderivative1 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f21[3]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative3(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f21[4]= Polderivative2 (bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);

}

else {

f1[1]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb],F[posx][posb],F[posx+1][posb]);

f1[2]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb],F[posx+1][posb],F[posx+2][posb]);

f1[3]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+1],F[posx+1][posb+1],F[posx+2][posb+1]);

f1[4]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+1],F[posx][posb+1],F[posx+1][posb]);

f2[1]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx][posb-1],F[posx][posb],F[posx][posb+1]);

f2[2]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+1][posb-1],F[posx+1][posb],F[posx+1][posb+1]);

f2[3]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+1][posb],F[posx+1][posb+1],F[posx+1][posb+2]);

f2[4]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx][posb],F[posx][posb+1],F[posx][posb+2]);

f12[1]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx-1][posb-1],F[posx-1][posb],F[posx-1][posb+1]),f2[1],f2[2]);

f12[2]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[1],f2[2],Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],F[posx+2][posb-1],F[posx+2][posb],F[posx+2][posb+1]));

f12[3]= Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],f2[4],f2[3],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx+2][posb],F[posx+2][posb+1],F[posx+2][posb+2]));

f12[4]= Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],F[posx-1][posb],F[posx-1][posb+1],F[posx-1][posb+2]),f2[4],f2[3]);

f21[1]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb-1],F[posx][posb-1],F[posx+1][posb-1]),f1[1],f1[4]);

f21[2]= Polderivative2(bGrid[posb-1],bGrid[posb],bGrid[posb+1],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb-1],F[posx+1][posb-1],F[posx+2][posb-1]),f1[2],f1[3]);

f21[3]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[2],f1[3],Polderivative2(xGrid[posx],xGrid[posx+1],xGrid[posx+2],F[posx][posb+2],F[posx+1][posb+2],F[posx+2][posb+2]));

f21[4]= Polderivative2(bGrid[posb],bGrid[posb+1],bGrid[posb+2],f1[1],f1[4],Polderivative2(xGrid[posx-1],xGrid[posx],xGrid[posx+1],F[posx-1][posb+2],F[posx][posb+2],F[posx+1][posb+2]));

f12[1] = 0.5*(f12[1]+f21[1]);
f12[2] = 0.5*(f12[2]+f21[2]);
f12[3] = 0.5*(f12[3]+f21[3]);
f12[4] = 0.5*(f12[4]+f21[4]);


}



return bicubint(f,f1,f2,f12,xGrid[posx],xGrid[posx+1],bGrid[posb],bGrid[posb+1],xA,bT);
}


